<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\Field\FieldFormatter\DatePoint;

use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'Plain' formatter for 'datetime' fields.
 *
 * @FieldFormatter(
 *   id = "date_point_plain",
 *   label = @Translation("Plain"),
 *   field_types = {"date_point"},
 * )
 */
final class PlainFormatter extends AbstractFormatter {

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\date_point\Plugin\Field\FieldType\DatePoint\DatePointItemList $items
   *
   * @return list<array>
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    foreach ($items as $delta => $item) {
      $elements[$delta] = [
        '#markup' => $item->getDatePoint()
          ->setTimezone($this->getTimezone())
          ->format($item->getFormat()),
        '#cache' => ['contexts' => ['timezone']],
      ];
    }
    return $elements;
  }

}
