<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\Field\FieldWidget\DatePoint;

use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\date_point\Data\Precision;
use Drupal\date_point\Plugin\Field\FieldType\DatePoint\DatePointItem;

/**
 * Base class for date point widgets.
 */
abstract class AbstractWidget extends WidgetBase {

  /**
   * {@selfdoc}
   */
  final protected function getTimezone(): \DateTimeZone {
    return new \DateTimeZone(\date_default_timezone_get());
  }

  /**
   * {@inheritdoc}
   *
   * @return array{value: string}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    $format = DatePointItem::buildFormat(
      Precision::from($this->getFieldSetting('precision')),
    );

    foreach ($values as &$value) {
      if (!$value['value']) {
        continue;
      }
      try {
        $value['value'] = (new \DateTimeImmutable($value['value'], $this->getTimezone()))
          ->setTimezone(new \DateTimeZone(DatePointItem::STORAGE_TIMEZONE))->format($format);
      }
      // @todo Catch \DateException once we drop support for PHP 8.3.
      catch (\Exception) {
        continue;
      }
    }
    return $values;
  }

}
