<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\Field\FieldWidget\DatePoint;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\date_point\Data\DatePointBehavior as Behavior;
use Drupal\date_point\Plugin\Field\FieldType\DatePoint\DatePointItemList;

/**
 * Plugin implementation of the 'date_point' widget.
 *
 * @FieldWidget(
 *   id = "date_point_textfield",
 *   label = @Translation("Plain"),
 *   field_types = {"date_point"},
 * )
 */
final class TextfieldWidget extends AbstractWidget {

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\date_point\Plugin\Field\FieldType\DatePoint\DatePointItemList $items
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    \assert($items instanceof DatePointItemList);

    $value = NULL;
    if ($items[$delta]->value) {
      $value = $items[$delta]->getDatePoint()
        ->setTimezone($this->getTimezone())
        ->format($items[$delta]->getFormat());
    }

    $element['value'] = $element + [
      '#type' => 'textfield',
      '#default_value' => $value,
      '#size' => 20,
      '#disabled' => $items->getBehavior() !== Behavior::MANUAL,
    ];

    return $element;
  }

}
