<?php

declare(strict_types=1);

namespace Drupal\date_point;

use Psr\Clock\ClockInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * A service for obtaining request time.
 */
final readonly class RequestClock implements ClockInterface {

  /**
   * {@selfdoc}
   */
  public function __construct(private ?RequestStack $requestStack = NULL) {}

  /**
   * {@selfdoc}
   */
  public function now(): \DateTimeImmutable {
    $request = $this->requestStack?->getCurrentRequest();
    $timestamp = $request ? $request->server->get('REQUEST_TIME_FLOAT') : $_SERVER['REQUEST_TIME_FLOAT'];
    // @todo Use \DateTimeImmutable::createFromTimeStamp() once we drop support
    // for PHP 8.3
    return \DateTimeImmutable::createFromFormat('U.u', (string) $timestamp);
  }

}
