<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript\DatePoint;

use Drupal\Tests\date_point\FunctionalJavascript\TestBase;

/**
 * @group date_point
 */
final class WidgetsTest extends TestBase {

  /**
   * {@selfdoc}
   */
  public function testTextfieldWidget(): void {
    $this->createField(
      type: 'date_point',
      widget: 'date_point_textfield',
      formatter: 'date_point_plain',
      storage_settings: ['precision' => 'milliseconds'],
    );

    $this->drupalGet('/node/add/page');
    $xpath = <<< 'XPATH'
      //label[text() = "Example"]
      /following-sibling::input
      [@type = "text" and @size = "20"]
      XPATH;
    $this->assertSession()->elementExists('xpath', $xpath);

    $this->createPage('2005-03-02 14:25:10.456');
    $this->assertRenderedValue('2005-03-02 14:25:10.456');
  }

  /**
   * {@selfdoc}
   */
  public function xxx(): void {
    $this->createField(
      type: 'date_point',
      widget: 'date_point_date_time_local',
      formatter: 'date_point_plain',
      storage_settings: ['precision' => 'milliseconds'],
    );

    $this->drupalGet('/node/add/page');
    $xpath = <<< 'XPATH'
      //label[text() = "Example"]
      /following-sibling::input
      [@type = "datetime-local" and @size = "20"]
      XPATH;
    $this->assertSession()->elementExists('xpath', $xpath);

    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/node/add/page');
    $driver->setValue('//input[@name = "title[0][value]"][1]', 'Test');
    // DrupalSelenium2Driver::setValue does not work for datetime-local inputs.
    $driver->evaluateScript(
      \sprintf(
        'document.querySelector(\'input[name="field_example[0][value]"]\').value = "%s";',
        '2005-03-02 14:25:10.456',
      ),
    );
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Page Test has been created.');
    $this->assertRenderedValue('2005-03-02 14:25:10.456');
  }

  /**
   * {@selfdoc}
   */
  public function testDatetimeLocalWidget(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();

    // -- Default configuration.
    $this->createField(
      type: 'date_point',
      widget: 'date_point_date_time_local',
      formatter: 'date_point_plain',
      storage_settings: ['precision' => 'milliseconds'],
    );
    $this->drupalGet('/node/add/page');
    $xpath = <<< 'XPATH'
      //label[text() = "Example"]
      /following-sibling::input
      [@type = "datetime-local" and @name = "field_example[0][value]" and @step = "0.001"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);

    $this->drupalGet('/node/add/page');
    $driver->setValue('//input[@name = "title[0][value]"][1]', 'Test');
    $this->setDateLocalValue('2005-03-02 14:25:10.456');
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Page Test has been created.');
    $this->assertRenderedValue('2005-03-02 14:25:10.456');

    // -- Field with seconds.
    $this->updateField(['precision' => 'seconds']);
    $this->drupalGet('/node/add/page');
    $xpath = <<< 'XPATH'
      //label[text() = "Example"]
      /following-sibling::input
      [@type = "datetime-local" and @name = "field_example[0][value]" and @step = "1"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);

    $this->drupalGet('/node/add/page');
    $driver->setValue('//input[@name = "title[0][value]"][1]', 'Test');
    $this->setDateLocalValue('2005-03-02 14:25:10');
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Page Test has been created.');
    $this->assertRenderedValue('2005-03-02 14:25:10');

    // -- Custom step.
    $this->openPageFormDisplayPage();
    $this->assertWidgetSummary('Step: auto');
    $this->openWidgetSettingsForm();
    $driver->setValue('//input[@name = "fields[field_example][settings_edit_form][settings][step]"]', '60');
    $this->submitWidgetSettingsForm();
    $this->assertWidgetSummary('Step: 60');
    $this->submitDisplayForm();

    $this->drupalGet('/node/add/page');
    $xpath = <<< 'XPATH'
      //label[text() = "Example"]
      /following-sibling::input
      [@type = "datetime-local" and @name = "field_example[0][value]" and @step = "60"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);

    $driver->setValue('//input[@name = "title[0][value]"][1]', 'Test');
    $this->setDateLocalValue('2005-03-02 14:25');
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Page Test has been created.');
    $this->assertRenderedValue('2005-03-02 14:25:00');
  }

  /**
   * {@selfdoc}
   */
  private function setDateLocalValue(string $value): void {
    // DrupalSelenium2Driver::setValue does not work for datetime-local inputs.
    $this->getSession()->getDriver()->evaluateScript(
      \sprintf('document.querySelector(\'input[name="field_example[0][value]"]\').value = "%s";', $value),
    );
  }

}
